//=============================================================================
// AM_Pickup.
//
// Author: Francesco Biscazzo
// Date: 2016
//=============================================================================
class AM_Pickup extends TournamentPickup;

var AdvancedMutator AM;
var Weapon OwnerWeapon;

var String oldOwnerState;
var String oldOwnerWeaponState;

function RegisterAdvancedMutator(AdvancedMutator AdvMut) {
	AM = AdvMut;
	SetCollision(false, false, false);
	SetCollisionSize(Owner.CollisionRadius, Owner.CollisionHeight );
	
	GotoState('Activated');
	Activate();
	
	SetTimer(0.1, True);
}

state Activated
{
	function Activate() {
		bActive = true;
		SetOwnerWeapon();
	}

	function EndState()
	{
		bActive = false;
		GotoState('Idle2');
	}

	function BeginState()
	{
		bActive = true;
		SetOwnerWeapon();	
	}
	
	// Use getStateName() on the weapon to get the fire type.
	function FireEffect() {
		AM.Fired(Pawn(Owner), OwnerWeapon, getAimedActor(Pawn(Owner), OwnerWeapon));
	}

	function OwnerJumped() {
		AM.Jumped(Pawn(Owner));
	}

	function ChangedWeapon()
	{
		local Weapon OldWeapon;

		OldWeapon = OwnerWeapon;
		super.ChangedWeapon();
		SetOwnerWeapon();
		
		AM.ChangedWeapon(Pawn(Owner), OldWeapon, OwnerWeapon);
	}

	function bool HandlePickupQuery( Inventory item )
	{
		AM.TouchedPickUp(Pawn(Owner), item);
	}
	
	function ZoneChange( ZoneInfo newZone ) {
		AM.ChangedZone(Pawn(Owner), newZone);
	}

	function SetOwnerWeapon()
	{
		if ( !bActive )
			return;

		// Make old weapon normal again.
		if ( OwnerWeapon != None )
		{
			OwnerWeapon.SetDefaultDisplayProperties();
			if ( OwnerWeapon.IsA('TournamentWeapon') )
				TournamentWeapon(OwnerWeapon).Affector = None;
		}
			
		OwnerWeapon = Pawn(Owner).Weapon;
		// Set new weapon as the OwnerWeapon.
		if ( OwnerWeapon != None )
		{
			if ( OwnerWeapon.IsA('TournamentWeapon') )
				TournamentWeapon(OwnerWeapon).Affector = self;
		}
	}

	function Timer()
	{
		local String OwnerState;
		local String OwnerWeaponState;
	
		SetLocation(Owner.Location);
	
		if (Owner.IsA('Pawn'))
				Pawn(Owner).bCountJumps = true;

		if ( !Pawn(Owner).bAutoActivate )
			OwnerJumped();
	
		OwnerState = String(Owner.getStateName());
		if (OwnerState != oldOwnerState) {
			AM.ChangedState(Pawn(Owner), OwnerState);
			oldOwnerState = OwnerState;
		}
		
		OwnerWeaponState = String(OwnerWeapon.getStateName());
		if (OwnerWeaponState != oldOwnerWeaponState) {
			AM.ChangedWeaponState(Pawn(Owner), OwnerWeapon, OwnerWeaponState);
			oldOwnerWeaponState = OwnerWeaponState;
		}
		
		AM.AimedActor(Pawn(Owner), OwnerWeapon, getAimedActor(Pawn(Owner), OwnerWeapon));
	}
}
	
static function actor getAimedActor(Pawn pawn, Weapon weapon) {
	local vector HitLocation, HitNormal, StartTrace, EndTrace, X,Y,Z;
	local float Accuracy;
	
	local rotator AdjustedAim; //To not to affect the pawn.Weapon.AdjustedAim
	
	Accuracy = 0.0;

	GetAxes(pawn.ViewRotation, X, Y, Z);
	StartTrace = pawn.Location + weapon.CalcDrawOffset() + weapon.FireOffset.X * X + weapon.FireOffset.Y * Y + weapon.FireOffset.Z * Z; 
	AdjustedAim = pawn.AdjustAim(1000000, StartTrace, 2*weapon.AimError, False, False);	
	EndTrace = StartTrace + Accuracy * (weapon.FRand() - 0.5 )* Y * 1000
		+ Accuracy * (weapon.FRand() - 0.5 ) * Z * 1000;
	X = vector(AdjustedAim);
	EndTrace += (10000 * X); 
	
	return pawn.TraceShot(HitLocation, HitNormal, EndTrace, StartTrace);
}